/**********************************************\
|                                              |
| ConsoleControl                               |
|                                              |
| Author: Sean Tasker                          |
|         slip@ice-d.com                       |
|         ice development                      |
|         www.ice-d.com                        |
|                                              |
| Date: 30-03-06                               |
|                                              |
| Description:                                 |
|   ConsoleControl is a virtual console class. |
|   It's designed to give control over where   |
|   characters will be displayed. It does this |
|   by buffering the contents of the console   |
|   and displaying all at once.                |
|                                              |
| Usage:                                       |
|   Simply create a ConsoleControl object by   |
|   specifying the width and height.           |
|                                              |
|   At most simple usage                       |
|     ConsoleControl *console;                 |
|     console->ConsoleControl(width,height);   |
|     console->printf("Hello World");          |
|     console->EnsureAlignment();              |
|     console->Display();                      |
|                                              |
|  ConsoleControl::Display() must be called    |
|  each time you want to update the display.   |
|  To ensure the buffer will be displayed      |
|  call ConsoleControl::EnsureAlignement()     |
|  to ensure the buffer is printed from *0,0*  |
|                                              |
|  There are various other functions provided  |
|  to assist in other operations. See below.   |
|                                              |
|  By specifying a width and/or height of 0    |
|  defaults will be used - see below for values|
|                                              |
\**********************************************/
#ifndef _CONSOLECONTROL_H_
#define _CONSOLECONTROL_H_
#include <iostream>
#include <stdio.h>
#include <stdarg.h>

#define DEFAULT_BUFFER_WIDTH 80				//The default console buffer size in windows
#define DEFAULT_BUFFER_HEIGHT 25


class ConsoleControl
{
public:
	ConsoleControl(int bufWidth, int bufHeight);
	~ConsoleControl();
	int GetSize() const;						//Size of buffer
	int GetWidth() const;						//Width of buffer
	int GetHeight() const;						//Height of buffer
	int GetCursorPosition() const;				//Get the current Cursor position
												//buffer is a 1D array, this returns the current
												//position in the buffer
	int GetCursorPositionX() const;				//The relative X position (colum) in the buffer
	int GetCursorPositionY() const;				//The relative Y position (row) in the buffer
	void MoveCursor(int x, int y);				//Move cursor to (colum, row)
	void MoveCursor(int pos);					//Move cursor to 1D index
	void MoveCursorVert(int a);					//Move the cursor vertically signed
	char *GetCol(int col) const;		//Not implemented yet
	char *GetRow(int row) const;				//Returns a pointer to a specific row
	char GetChar(int pos) const;				//Get a character at index
	char GetChar(int x, int y) const;			//Get a character at (col, row)
	void printf(const char* msg, ...);			//printf - as normal
	void print(char *msg);						//Prints a char array to the buffer
	void print(char c);							//Prints a char to the buffer
	void Display();								//Prints the buffer to the console (cout)
	void DisplaySeg(int start, int finish);		//Display a segment of the buffer at current position
												//in console. (cout)
	char SingleCharInput(int x, int y);	//Not implemented correctly
	void ClearBuffer();							//Clear the buffer to ' 's
	void ClearRow(int row);						//Clear a specific row
	void ClearColum(int col);					//Clear a colum
	void EnsureAlignment();						//Ensures the buffer when displayed (directly after)
												//will display correctly in the buffer
private:
	int cp;										//Cursor Position (index)
	int cx;										//Cursor X (col)
	int cy;										//Cursor Y (row)
	int bufferWidth;							//Width of the buffer
	int bufferHeight;							//Height of the buffer
	int bufferSize;								//Size of the buffer
	char *buffer;								//The buffer
	bool bufferCreateError;						//If the buffer was not created this is set to true
												//Error checking would be as follow
												//
												//ConsoleControl *console=new ConsoleControl(80,25);
												//if(console->bufferCreateError)
												//		;//ConsoleControl object invalid
	void MoveCp();								//Move the cursor 1 in the buffer

};

#endif
